dojo.provide("com.ibm.pb.data.ResourceLoader");

dojo.require("com.ibm.mm.builder.DelegatedDeferredImpl");
dojo.require("com.ibm.pb.logging.Loggable");

/**
 * Reusable generic resource loader that returns a Deferred object that loads a resource
 * remotely.  Underneath, the loader pipes all requests to the same resource through the
 * same IO operation so that multiple simultaneous requests for the same data aren't made
 * at the same time, but rather piggy back on top of an existing operation.  Supports
 * loading in async or sync mode. 
 */
dojo.declare("com.ibm.pb.data.ResourceLoader", com.ibm.pb.logging.Loggable, {
	
	_cache: {},
	statusCodes: com.ibm.mm.enabler.model.HttpStatusCodes,
	
	constructor: function() {
		this._cache = {};
	},
	
	getData: function(url, type) {
		// multiple calls to this before a previous call finishes may result in multiple requests for the same data.... depending on
		// the implementation of the ConfigService Deferred
		var mname = "getData", lgr = this.LOGGER, me = this;
		if(!this._cache[url]) {
			this._cache[url] = new com.ibm.mm.builder.DelegatorDeferredImpl(this, function(deferred, sync, params) {
				var callBack = deferred.getFinishedCallback(), callBackParams = deferred.getFinishedCallbackParameters();
				lgr.trace(mname, "Start loading remote data");
				if (deferred._data) {
					lgr.trace(mname, "Data already loaded ${0}", [deferred._data]);
					if(callBack) {
						callBack(deferred._data, 200, callBackParams);
					}
				}
				else {
					dojo.xhrGet({
						url: url,
						load: function(data, ioArgs){
							lgr.trace(mname, "Received response from url ${0} with data ${1}", [url, data]);
							deferred._hasFired = true;
							me._finishDeferred(deferred, data, 200);
						},
						error: function(err, ioArgs) {
							lgr.warning(mname, "Error getting data from url ${0}: ${1}", [url, err]);
							if(!deferred._hasFired) {
								// we don't want to get into this if the deferred's callback was
								// already fired in the load function, but threw an error that dojo
								// catches underneath us and then calls the error function too
								me._finishDeferred(deferred, err, me.statusCodes.HTTP_NOT_FOUND);
							}
						},
						handleAs: type,
						sync: sync
					});
				}
				return deferred._data;
	        });
		}
		var dfd = new com.ibm.mm.builder.DelegatedDeferredImpl();
		this._cache[url].addDelegate(dfd);
		return dfd;
	},
	_finishDeferred: function(deferred, data, status) {
		deferred._data = data;
		var callBack = deferred.getFinishedCallback(), callBackParams = deferred.getFinishedCallbackParameters();
		if(callBack) {
			callBack(data, status, callBackParams);
		}
		return data;
	}
});


com.ibm.pb.data.RESOURCE_LOADER = new com.ibm.pb.data.ResourceLoader();
