// @author Laura
// module to add runtime style filters to each layout container on the page

dojo.provide("com.ibm.pb.decorations.LayoutFilters");
dojo.require("com.ibm.pb.layout.LayoutParser");
dojo.declare("com.ibm.pb.decorations.LayoutFilters", com.ibm.pb.logging.Loggable, {

	constructor: function() {
		// summary: assigns runtime style filters to each of the given containers
		var mname = "constructor", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		this.layoutParser = com.ibm.pb.layout.LayoutParser;
		this._filters = [];
		this.filterAdded = false;
		
		this.evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		this.changePageModeEvt = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "_onPageModeChanged");
		
		// #19538, when click save, the filters will be destroyed and re-constructed
		// in this case _addStyleFilter again
		if (com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode() == "edit") {
			if (this.filterAdded) return;
			this.filterAdded = true;
			this._addStyleFilter();	
		}
		lgr.exiting(mname);
	},
	
	_onPageModeChanged: function(iEvent){
		if (this.filterAdded) return;
		if(iEvent == "edit") {
			this.filterAdded = true;
			this._addStyleFilter();	
		}		
	},
	
	_addStyleFilter: function() {
		// initialize layout and runtime skin models
		var containers = this.layoutParser.findContainers(this.layoutParser.getRootNode());
		this.rsModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID(),
			navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(),
			layoutModel = navModel.getLayoutModel(pageId),
			c = ibmCfg.themeConfig.layoutFilterDefinitions;  // a map that pairs css classes on container types with their corresponding layout filter
		
		// loop through the containers and apply the appropriate runtime style filter to each
		for (var i = 0, l = containers.length; i < l; i++) {
			// loop through the filter map to find the correct filter class to apply
			for (var j = 0, cl = c.length; j < cl; j++) {
				if (dojo.hasClass(containers[i], c[j].id)) {
					// get an instance of the filter class needed for this container
					var filterCls = dojo.getObject(c[j].filter);
					if (!filterCls) {
						dojo["require"](c[j].filter);
						filterCls = dojo.getObject(c[j].filter);
						if (!filterCls) {
							this.logger.severe(mname,"Cannot find layout container filter class",[arguments]);
							return;
						}
					}
					var filter = new filterCls();
					// find the container in the layout model and apply the filter to it
					var id = this.layoutParser.extractID(containers[i]);
					var layoutContainer = layoutModel.find(id).start();
					var intt = this.rsModel.addStyleFilter(com.ibm.mashups.enabler.runtime.skin.Constants.ORDINAL_FIRST, filter, layoutContainer);
					this._filters.push({"id":intt,"filter":filter});
				}
			}
		}	
	},
	
	destroy: function() {
		// summary: remove and destroy all the style filters
		var mname = "destroy", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		this.evtSvc.unsubscribeEvent(this.changePageModeEvt);		
		var len = this._filters.length;
		for (var i = 0; i < len; i++) {
			this.rsModel.removeStyleFilter(this._filters[i].id);
			delete this._filters[i].filter;
		}
		this._filters = [];
		
		lgr.exiting(mname);
	}
});
