dojo.provide("com.ibm.pb.tree.DFSTraversal");

dojo.require("com.ibm.pb.tree.Traversal");

dojo.declare("com.ibm.pb.tree.DFSTraversal", com.ibm.pb.tree.Traversal, {
	next: function(node, test, callback, stop, params) {
		// visit the next node in the tree using depth-first search
		var mname = "next", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var iterator = null;
		if(this.model.hasChildren(node)) {
			lgr.trace(mname, "Traversing down children of ${0}", [node]);
			var dfdParams = {
				node: node,
				test: test,
				callback: callback,
				stop: stop,
				params: params,
				list: []
			}, me = this;
			this.model.getChildren(node)
				.setForEachCallback(function(){
					me._collectChild.apply(me, arguments);
				}, dfdParams)
				.setFinishedCallback(function(){
					me._visitChildren.apply(me, arguments);
				}, dfdParams)
				.start(false);
		}
		else {
			lgr.trace(mname, "Reached end of traversal at ${0}", [node]);
			if(!this.isStopped()) {
				callback(params.list);
			}
		}
		lgr.exiting(mname, arguments);
	},
	_collectChild: function(resource, dfdParams) {
		dfdParams.list.push(resource);
	},
	_visitChildren: function(resource, status, dfdParams) {
		var mname = "_visitChildren", lgr = this.LOGGER, me = this;
		lgr.entering(mname, [resource, status, dfdParams]);
		if((status !== 200) && (status != "200")) {
			lgr.severe(mname, "An error occurred ${0} ${1}", [status, resource]);
			lgr.exiting(mname, arguments);
			return;
		}
		var subTreeCallback = function() {
			if(!me.isStopped()) {
				var currentNode = dfdParams.list.shift();
				if(currentNode) {
					me._visitChild(currentNode, dfdParams.test, subTreeCallback, dfdParams.stop, dojo.delegate(dfdParams.params, {level: dfdParams.params.level+1}));
				}
				else {
					lgr.trace(mname, "Reached end of list of children of ${0}", [dfdParams.node]);
					dfdParams.callback(dfdParams.params.list);
				}
			}
			else {
				lgr.trace(mname, "Stopped in the middle of the list of children of ${0}", [dfdParams.node]);
				dfdParams.callback(dfdParams.params.list);
			}
		}
		lgr.exiting(mname, arguments);
		subTreeCallback();
	},
	_visitChild: function(node, test, callback, stop, params) {
		var mname = "_visitChild", lgr = this.LOGGER, me = this;
		lgr.trace(mname, "Visiting next node in list of children ${0}", [node]);
		if(!this.isStopped()) {
			this.visit(node, test, callback, stop, params);
		}
		else {
			// FIXME: Stop any iterators at this point, 13729
			lgr.trace(mname, "Hard stopped iterating");
		}
	}
});