/* Copyright IBM Corp. 2013  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataFilter");

dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.Store");

dojo.declare("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataFilter", null, {

	constructor: function(args){
		
		dojo.mixin(this, args);
		
		this.typeList = []
		for (var i=0; i<this.data.columnList.length; i++){
			this.typeList.push('text');
		}
			
		this.parseColumns(this.data, this.columnLabel, this.columnSorttype);
		
		this.addDataFilter(this.filterColumn,this.filterFunction);
		
		this.data = this.data.result;
		
		this.store = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.Store(this.data);
		
	},
	
	clearDataFilters: function(){
		dojo.forEach(this.columnList, function(colList){
			colList.filterFunction = null;
		}, this);
    },
	
	addDataFilter: function(column, filterFunction){
        this.clearDataFilters();        
        this.setDataFilter(column, dojo.hitch(this, filterFunction));
    },
	
	setDataFilter: function(/* string */field, /* function */ fn){
        //	summary
        //	set a filtering function on the passed field.
		this.dataFilterIsOn = true;
        for (var i = 0; i < this.columnList.length; i++) {
            if (this.columnList[i].label === field) {
                this.columnList[i].filterFunction = fn;
                break;
            }
        }
    },
	
	
	parseColumns: function(){
	
		 this.columnList = [];
        
        for (var i = 0; i < this.data.columnList.length; i++) {
        
            var columnListItem = {};
            columnListItem["label"] = this.data.columnList[i];//columnLabel;
            
            columnListItem.filterFunction = null;
            this.columnList.push(columnListItem);
            
        }
	},
	
	filterData: function() {
		if (this.filterTargetValue) {
			if (this.isFilter) {
				this.applyDataFilter(false);
			} else if (this.isHighLightFilter) {
	    		this.applyHighlightFilter();
				
			} else if (this.isRemoveFilter) {
				this.applyDataFilter(true);
			}
		}
		return this.store;
	},
	
	applyDataFilter: function(reverse){
		// summary
		// Filter data. when reverse is true, reverse selection
		var newData = [];
        for (var i = 0, dl = this.data.length; i < dl; i++) {
            var b = true;
            var row = this.data[i];
            for (var j = 0, cl = this.columnList.length; j < cl; j++) {
            	var cell = this.columnList[j];
                var columnName = cell.label;
                var value = row[columnName];
                if (this.typeList[j] === "Date" && value && !value.getYear) {
                    value = new Date(value);
                }
                if (cell.filterFunction && !cell.filterFunction(value)) {
                    b = false;
                    break;
                }
            }
            if (b ^ reverse) newData.push(this.data[i]);
        }
        this.data = newData;
		this.store.setData(newData);
		
	},
	
	applyHighlightFilter: function(){
    	this.resetHighlight();
        var data = this.store.get(true);
		
        for (var i = 0, dl = this.data.length; i < dl; i++) {
            var b = true;
            var row = this.data[i];
            for (var j = 0, cl = this.columnList.length; j < cl; j++) {
                var columnName = this.columnList[j].label;
                var value = row[columnName];
                if (this.typeList[j] === "Date" && value && !value.getYear) {
                    value = new Date(value);
                }
				if (this.columnList[j].filterFunction && !this.columnList[j].filterFunction(value)) {
                    b = false;
                    break;
                }
            }
            if (b && this.filterTargetValue && this.dataFilterIsOn) {
            	data[i].isHighlighted = true;
            }
        }

	},
	
	resetHighlight: function(){
        this.store.forEach(function(element){
            element.isHighlighted = false;
        });
    },
	
	//------------------------------------------------------------------------
    //Compare function set
    bw: function(name){
        return (name.indexOf(this.filterTargetValue) === 0);
    },
    
    ct: function(name){
        return (name.indexOf(this.filterTargetValue) !== -1);
    },
    
    ew: function(name){
        var len = this.filterTargetValue.length;
        var temp = name.substring(name.length - len, name.length);
        return (temp === this.filterTargetValue);
    },
    
    gt: function(number){
        return (dojo.number.parse(number) > dojo.number.parse(this.filterTargetValue));
    },
    
    eq: function(number){
        return (dojo.number.parse(number) === dojo.number.parse(this.filterTargetValue));
    },
    
    lt: function(number){
        return (dojo.number.parse(number) < dojo.number.parse(this.filterTargetValue));
    },
    
    notlt: function(number){
        return (this.gt(number) || this.eq(number));
    },
    
    notgt: function(number){
        return (this.lt(number) || this.eq(number));
    }
    
    //-----------------------------------------------------------------------
	
});
